@@ -13,7 +13,7 @@ from pdfstitcher.layerfilter import LayerFilter
 from pdfstitcher.pagefilter import PageFilter
 from pdfstitcher import utils
 from pdfstitcher.utils import Config
-from pdfstitcher.ui.dialogs import PrefsDialog, UpdateDialog
+from pdfstitcher.ui.dialogs import PrefsDialog, UpdateDialog, BugReporter
 from pdfstitcher.ui.io_tab import IOTab
 from pdfstitcher.ui.tile_tab import TileTab
 from pdfstitcher.ui.layers_tab import LayersTab
@@ -170,7 +170,7 @@ class PDFStitcherFrame(wx.Frame):
             trim[1] = utils.txt_to_float(self.tt.right_trim_txt.GetValue())
             trim[2] = utils.txt_to_float(self.tt.top_trim_txt.GetValue())
             trim[3] = utils.txt_to_float(self.tt.bottom_trim_txt.GetValue())
-            self.tiler.set_trim(trim)
+            self.tiler.trim = trim
             self.tiler.actually_trim = bool(self.tt.trim_overlap_combo.GetSelection())
             self.tiler.override_trim = self.tt.override_trim.GetValue()
 
@@ -205,7 +205,8 @@ class PDFStitcherFrame(wx.Frame):
             if do_tile:
                 self.tiler.in_doc = filtered
                 new_doc = self.tiler.run(rows, cols)
-                print(_("Tiling successful"))
+                if new_doc:
+                    print(_("Tiling successful"))
             else:
                 # extract the requested pages
                 page_filter = PageFilter(filtered)
@@ -219,8 +220,9 @@ class PDFStitcherFrame(wx.Frame):
             return
 
         try:
-            new_doc.save(self.out_doc_path)
-            print(_("Successfully written to") + " " + self.out_doc_path)
+            if new_doc:
+                new_doc.save(self.out_doc_path)
+                print(_("Successfully written to") + " " + self.out_doc_path)
         except Exception as e:
             print(
                 _("Something went wrong") + ", " + _("unable to write to") + " " + self.out_doc_path
@@ -283,10 +285,25 @@ class PDFStitcherFrame(wx.Frame):
         about_item = help_menu.Append(wx.ID_ABOUT)
         self.Bind(wx.EVT_MENU, self.on_docs, docs_item)
         self.Bind(wx.EVT_MENU, self.on_about, about_item)
+        bug_item = wx.MenuItem(
+            help_menu,
+            wx.ID_ANY,
+            text=_("Report a bug"),
+            helpString=_("Open the dialog to report a bug"),
+        )
+        help_menu.Append(bug_item)
+        self.Bind(wx.EVT_MENU, self.on_bug, bug_item)
         menu_bar.Append(help_menu, "&" + _("Help"))
 
         self.SetMenuBar(menu_bar)
 
+    def on_bug(self, event):
+        """
+        Open the dialog to report a bug.
+        """
+        bug_dia = BugReporter(parent=self)
+        bug_dia.Show()
+
     def on_prefs(self, event):
         """
         Create and show the preferences dialog.
@@ -344,7 +361,6 @@ class PDFStitcherFrame(wx.Frame):
             wildcard="PDF files (*.pdf)|*.pdf",
             style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT,
         ) as fileDialog:
-
             if fileDialog.ShowModal() == wx.ID_CANCEL:
                 return
 
@@ -399,7 +415,6 @@ class PDFStitcherFrame(wx.Frame):
             wildcard="PDF files (*.pdf)|*.pdf",
             style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST,
         ) as fileDialog:
-
             if fileDialog.ShowModal() == wx.ID_CANCEL:
                 return
 
@@ -462,10 +477,18 @@ class PDFStitcherFrame(wx.Frame):
 
         else:
             print(_("PDF file loaded without errors."))
+            # Check for permissions
+            if self.in_doc.is_encrypted:
+                permissions = self.in_doc.allow
+                print(_("Warning: this PDF is encrypted with the following permissions:"))
+                for perm, allowed in permissions._asdict().items():
+                    print(f"{perm}: {allowed}")
+
+            print(_("Please be respectful of the author and only use this tool for personal use."))
             Config.general["open_dir"] = str(Path(pathname).parent)
 
 
-def main():
+def main(language_warning: str):
     """
     Create the app and run it.
     """
@@ -484,6 +507,9 @@ def main():
     frm.SetSize(frm.FromDIP(app_size))
     frm.reset_sash_position()
 
+    if language_warning:
+        print(language_warning)
+
     frm.Show()
 
     if Config.general["check_updates"]:
